<?php

namespace Drupal\ldap_sso;

use Drupal\Core\Url;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\KernelEvents;
use Symfony\Component\HttpKernel\Event\GetResponseEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\Cookie;

/**
 * Provides a MyModuleSubscriber.
 */
class LdapSsoBootSubscriber implements EventSubscriberInterface, LdapSsoInterface {

  private $detailedWatchdogLog;

  /**
   * Fetches debugging level.
   */
  public function __construct() {
    $this->detailedWatchdogLog = \Drupal::config('ldap_help.settings')->get('watchdog_detail');
  }

  /**
   * Determine if we should attempt SSO.
   */
  public function checkSsoLoad(GetResponseEvent $event) {

    if ((PHP_SAPI === 'cli') || \Drupal::currentUser()->isAnonymous() == FALSE) {
      if ($this->detailedWatchdogLog) {
        \Drupal::logger('ldap_sso')->debug('CLI or logged in user');
      }
      return;
    }
    elseif ($this->checkExcludePath()) {
      if ($this->detailedWatchdogLog) {
        \Drupal::logger('ldap_sso')->debug('Excluded path');
      }
      return;
    }
    elseif (!(isset($_COOKIE['seamless_login'])) || $_COOKIE['seamless_login'] == $this::AUTO_LOGIN_YES) {
      if ($this->detailedWatchdogLog) {
        \Drupal::logger('ldap_sso')->debug('SSO Boot assumed');
      }
      $this->assumeSsoBoot();
    }
    else {
      if ($this->detailedWatchdogLog) {
        \Drupal::logger('ldap_sso')->debug('Anonymous and no SSO boot.');
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[KernelEvents::REQUEST][] = ['checkSsoLoad', 20];
    return $events;
  }

  /**
   * Continue booting assuming we are doing SSO.
   */
  private function assumeSsoBoot() {
    $path_args = explode('/', \Drupal::service('path.current')->getPath());

    if ($path_args[0] == 'user' && !(is_numeric($path_args[1]))) {
      return;
    }
    elseif ($path_args[0] == 'user' && $path_args[1] == 'logout') {
      return;
    }
    else {
      if ($this->detailedWatchdogLog) {
        \Drupal::logger('ldap_sso')->debug('Path is valid');
      }

      if (isset($_COOKIE['seamless_login_attempted'])) {
        if ($this->detailedWatchdogLog) {
          \Drupal::logger('ldap_sso')->debug('Cookie present');
        }
        $login_attempted = $_COOKIE['seamless_login_attempted'];
      }
      else {
        $login_attempted = 'false';
      }
      $config = \Drupal::config('ldap_authentication.settings')->get('ldap_authentication_conf');

      if (isset($config['seamlessLogin']) &&
        $config['seamlessLogin'] == 1 &&
        $login_attempted == 'false') {

        if ($this->detailedWatchdogLog) {
          \Drupal::logger('ldap_sso')->debug('Transferring to login controller');
        }

        if (isset($config['cookieExpire']) && $config['cookieExpire'] == -1) {
          // Length of session.
          $cookie_lifetime = 0;
        }
        else {
          // A value quickly in the past.
          $cookie_lifetime = time();
        }

        $cookies[] = new Cookie('seamless_login_attempted', 'true', $cookie_lifetime, base_path());

        // This is set to destination() since the request uri is usually system/40x already.
        $original_path = \Drupal::destination()->get();
        $response = new RedirectResponseWithCookie(Url::fromRoute('ldap_sso.login_controller')->toString() . '?destination=' . $original_path, 302, $cookies);
        $response->send();
        exit(0);
      }
      else {
        return Response::create('', 403);
      }
    }
  }

  /**
   * Exclude default excluded paths.
   */
  private function defaultPathsToExclude() {
    return [
      '/admin/config/search/clean-urls/check',
      '/user/login/sso',
    ];
  }

  /**
   * Check to exclude paths from SSO.
   *
   * @param bool|string $path
   *   Path to check for exclusion.
   *
   * @return bool
   *   Path excluded or not.
   */
  private function checkExcludePath($path = FALSE) {

    $result = FALSE;
    if ($path) {
      // don't derive.
    }
    elseif ($_SERVER['PHP_SELF'] == '/index.php') {
      $path = \Drupal::service('path.current')->getPath();
    }
    else {
      // cron.php, etc.
      $path = ltrim($_SERVER['PHP_SELF'], '/');
    }

    if (in_array($path, $this->defaultPathsToExclude())) {
      return TRUE;
    }

    if (isset($config['ssoExcludedHosts']) && is_array($config['ssoExcludedHosts'])) {
      $host = $_SERVER['SERVER_NAME'];
      foreach ($config['ssoExcludedHosts'] as $host_to_check) {
        if ($host_to_check == $host) {
          return TRUE;
        }
      }
    }

    if (isset($config['ssoExcludedPaths'])) {
      $patterns = implode("\r\n", $config['ssoExcludedPaths']);
      if ($patterns) {
        if (function_exists('drupal_get_path_alias')) {
          $path = drupal_get_path_alias($path);
        }
        $path = (function_exists('drupal_strtolower')) ? drupal_strtolower($path) : strtolower($path);

        $to_replace = [
          // Newlines.
          '/(\r\n?|\n)/',
          // Asterisks.
          '/\\\\\*/',
          // <front>.
          '/(^|\|)\\\\<front\\\\>($|\|)/',
        ];
        $replacements = [
          '|',
          '.*',
          '\1' . preg_quote(\Drupal::config('system.site')->get('frontpage'), '/') . '\2',
        ];
        $patterns_quoted = preg_quote($patterns, '/');
        $regex = '/^(' . preg_replace($to_replace, $replacements, $patterns_quoted) . ')$/';
        $result = (bool) preg_match($regex, $path);
      }
    }

    return $result;
  }

}
